@@ -459,12 +459,12 @@ class ClusterAnalysis(object):
 
         # plot population with labels
         if population_freq.index.size < 40:
-            texts = [plt.text(x=coordinates[i, 0]+0.008, y=coordinates[i, 1]-0.01, s=population_freq.index.tolist()[i], size=15) for i in range(len(matrix))]
+            texts = [plt.text(coordinates[i, 0], coordinates[i, 1], s=population_freq.index.tolist()[i], size=15) for i in range(len(matrix))]
             import warnings
             warnings.filterwarnings('ignore')
             adjust_text(texts)
         else:
-            self.logger.warning('[Y-LineageTracker] Too much population in population file')
+            self.logger.warning('[Y-LineageTracker] Too much population in the population file')
 
         # output figure
         output_fig_file = output_set[0]
