@@ -29,20 +29,80 @@ class CommonData(object):
 
         return region_info
 
-    def read_ref_info(self, snp_only, approximate):
+    def read_ref_info(self, snp_only, approximate, ref):
 
-        ref_info = pd.read_csv(self.data_path + 'ISOGG_haplogroup_info.csv',
-                               header=0,
-                               sep=',',
-                               encoding='utf-8',
-                               dtype='object')
+        if ref:
+            import numpy as np
+            from GetConfig import getConfig
+            config = getConfig()
+            common_trunk = config.get('HaplogroupTree', 'CommonTrunk').split(',')
+
+            def get_approx(x):
+                if x['Haplogroup'].endswith('~'):
+                    if x['Haplogroup'].strip('~') in hgs:
+                        return np.nan
+                    else:
+                        return 'Approximate'
+                else:
+                    return np.nan
+
+            def get_approx_hap(x):
+                if not x['Haplogroup'].endswith('~'):
+                    return x['Haplogroup']
+                else:
+                    if pd.isna(x['ApproximateInfo']):
+                        return x['Haplogroup']
+                    else:
+                        return x['Haplogroup'].strip('~')
+
+            def get_main(x):
+                if len(x['Haplogroup']) == 1:
+                    return 'Main'
+                elif x['Haplogroup'] in common_trunk:
+                    return 'Main'
+                else:
+                    return np.nan
+
+            # columns in custom data
+            required_columns = ['Mutation', 'Haplogroup', 'Pos', 'Variant']
+            # read custom data
+            ref_info = pd.read_csv(ref,
+                                   header=0,
+                                   sep='\t',
+                                   encoding='utf-8',
+                                   dtype='object')
+            if ref_info.columns.tolist() != required_columns:
+                raise KeyError('Header of ref file not correct, please check the header is: Mutation, Haplogroup, Pos, Variant')
+
+            # fill data
+            ref_info['rs'] = '.'
+            ref_info['BuildNa'] = '.'
+            ref_info['MutationInfo'] = '.'
+            ref_info['KeyInfo'] = '.'
+            ref_info['ReferenceInfo'] = '.'
+            hgs = sorted(set(ref_info[ref_info['Haplogroup'].map(lambda x: not x.endswith('~'))]['Haplogroup'].tolist()))
+            ref_info['ApproximateInfo'] = ref_info.apply(get_approx, axis=1)
+            ref_info['Haplogroup'] = ref_info.apply(get_approx_hap, axis=1)
+            ref_info['MainInfo'] = ref_info.apply(get_main, axis=1)
+
+            ref_info.rename(columns={'Pos': 'BuildRef'}, inplace=True)
+
+            ref_info = ref_info[['Mutation', 'Haplogroup', 'rs', 'BuildRef', 'BuildNa', 'Variant',
+                                 'MutationInfo', 'KeyInfo', 'ReferenceInfo', 'ApproximateInfo', 'MainInfo']]
+        else:
+            ref_info = pd.read_csv(self.data_path + 'ISOGG_haplogroup_info.csv',
+                                   header=0,
+                                   sep=',',
+                                   encoding='utf-8',
+                                   dtype='object')
         if not approximate:
-            ref_info = ref_info[ref_info['ApproximateInfo']!='Approximate']
+            ref_info = ref_info[ref_info['Haplogroup'].map(lambda x: not x.endswith('~'))]
+        approximate_haps = sorted(set(ref_info[ref_info['ApproximateInfo']=='Approximate']['Haplogroup']))
         if snp_only:
             ref_info = ref_info[ref_info['Variant'].map(lambda x: len(x) == 1)]
         ref_ind_info = ref_info[ref_info['ReferenceInfo']=='Ref']
 
-        return ref_info, ref_ind_info
+        return ref_info, ref_ind_info, approximate_haps
 
 
     def read_pop_info(self):
@@ -84,7 +144,7 @@ class CommonData(object):
 
     def read_tree_info(self):
 
-        tree_file = open(self.data_path + 'haplogroup_tree_2018.txt', 'r+')
+        tree_file = open(self.data_path + 'haplogroup_tree_2018.txt', 'r')
         tree_info = list(map(lambda x: x.replace('\n', ''), tree_file.readlines()))
         tree_file.close()
 
@@ -154,8 +214,21 @@ def check_cutoff(filter, format):
     return cutoff
 
 
+# check reference marker file
+def check_ref_marker(ref, build):
+
+    if not ref:
+        if build == 'Ref':
+            raise argparse.ArgumentTypeError('No reference marker file specified')
+    else:
+        if build != 'Ref':
+            logger = logging.getLogger()
+            logger.warning('[Y-LineageTracker] [Warning] Not specify build argument Ref, use %d' % build)
+
+    return ref, build
+
 # check bam or vcf input
-def check_BAM_VCF_input(vcf, bam):
+def check_BAM_VCF_input(vcf, bam, cram):
 
     if vcf:
         input = vcf
@@ -165,8 +238,13 @@ def check_BAM_VCF_input(vcf, bam):
         else:
             format = 'vcf'
     else:
-        input = ', '.join(bam)
-        format = 'bam'
+        if bam:
+            input = ', '.join(bam)
+            format = 'bam'
+        else:
+            input = ', '.join(cram)
+            format = 'cram'
+
 
     return input, format
 
@@ -235,6 +313,8 @@ def check_hap_input(input_file, type, format='matrix'):
     if type == 'haplogroup':
         if input_data.columns.size != 2 or input_data.columns.tolist() != ['SampleID', 'Haplogroup']:
             raise KeyError('Header of haplogroup file not correct, please check the header is: SampleID, Haplogroup')
+        else:
+            input_data['Haplogroup'] = input_data['Haplogroup'].map(lambda x: x.strip('~'))
     elif type == 'haplotype':
         if input_data.columns.size < 2 or input_data.columns.tolist()[0] != 'SampleID':
             raise KeyError('Header of haplotype file not correct, please check the header is: SampleID, Haplotype Names')
