@@ -21,18 +21,19 @@ class TrackLineage(object):
     3. Output result of classification
     '''
 
-    def __init__(self, matched_info, ref_ind_info, build, format, chip):
+    def __init__(self, matched_info, ref_ind_info, approximate_haps, build, format, chip):
 
         # record running log
         self.logger = logging.getLogger()
 
         # get reference haplogroups of which all indivisuals are same
-        if build == 37:
+        if build == '37' or 'Ref':
             idx = -7
-        elif build == 38:
+        elif build == '38':
             idx = -8
 
         # get parameters from config file
+        self.build = build
         self.marker_rate = float(config.get('HaplogroupTree', 'MarkerRate'))
         self.length_cutoff = config.getint('HaplogroupTree', 'LengthCutoff')
         self.matched_marker = config.get('HaplogroupMarker', 'MatchMarker')
@@ -40,16 +41,15 @@ class TrackLineage(object):
         self.track_symbol = config.get('HaplogroupMarker', 'TrackSymbol') # track symbol is '->'
         self.track_rate = float(config.get('HaplogroupTree', 'TrackRate'))
         self.tree_trunk = config.get('HaplogroupTree', 'CommonTrunk').split(',')
+        self.approximate_haps = approximate_haps
         self.haplogroup_interval = config.getint('HaplogroupTree', 'HaplogroupInterval')
 
-        if format == 'vcf' and not chip:
-            self.add_common = True
+        if format == 'vcf' and not chip and build != 'Ref':
             common_sites = [i for i in ref_ind_info['Build'+str(build)].tolist() if i not in matched_info[:, idx].tolist()]
             common_df = ref_ind_info[ref_ind_info['Build'+str(build)].map(lambda x: x in common_sites)]
             self.common_haps = list(set(common_df['Haplogroup'])&set(self.tree_trunk))
             self.common_key_haps = list(set(common_df[common_df['KeyInfo']=='Key']['Haplogroup'])&set(self.tree_trunk))
         else:
-            self.add_common = False
             self.common_key_haps = []
             self.common_haps = []
 
@@ -125,12 +125,11 @@ class TrackLineage(object):
 
         traversed = []
         for hap, matched, total in downstream:
-            if hap.startswith(final_hap):
-                length = len(hap)
-                final_hap = hap
-                rate = matched/total
-                if final_hap in key_haps:
-                    key_hap = final_hap
+            length = len(hap)
+            final_hap = hap
+            rate = matched/total
+            if final_hap in key_haps and len(hap)-length < self.haplogroup_interval:
+                key_hap = final_hap
             else:
                 if matched/total > rate and len(hap)-length < self.haplogroup_interval:
                     length = len(hap)
@@ -253,9 +252,18 @@ class TrackLineage(object):
             track = final_hap
         else:
             track = ''
+            alpha = not final_hap[-1].isalpha()
             for num_str in range(len(final_hap)):
-                up = final_hap[0: len(final_hap)-num_str]
+                tmp_up = final_hap[0: len(final_hap)-num_str]
+                is_alpha = tmp_up[-1].isalpha()
+                if is_alpha != alpha:
+                    up = tmp_up
+                else:
+                    continue
+                if up in self.approximate_haps:
+                    up = up + '~'
                 track = track + self.track_symbol + up
+                alpha = is_alpha
             track = track[2:]
             # A is special haplogroup in name
             if track[-1] == config.get('HaplogroupA', 'SpecificHaplogroup'):
@@ -296,11 +304,15 @@ class TrackLineage(object):
         for hap in haps_in_track:
             if hap in matched_haps:
                 hap_index = matched_haps.index(hap)
-                matched = matched_haps_freq[hap_index]
-                total = total_haps_freq[hap_index]
-                hap_without_count = self.track_symbol + hap + self.track_symbol
-                hap_with_count = self.track_symbol + hap + '(' + str(matched) + '/' + str(total) + ')' + self.track_symbol
-                track = track.replace(hap_without_count, hap_with_count)
+            elif hap.strip('~') in matched_haps:
+                hap_index = matched_haps.index(hap.strip('~'))
+            else:
+                continue
+            matched = matched_haps_freq[hap_index]
+            total = total_haps_freq[hap_index]
+            hap_without_count = self.track_symbol + hap + self.track_symbol
+            hap_with_count = self.track_symbol + hap + '(' + str(matched) + '/' + str(total) + ')' + self.track_symbol
+            track = track.replace(hap_without_count, hap_with_count)
         track = track[2:]
 
         return track
@@ -328,13 +340,13 @@ class TrackLineage(object):
 
         # initial variables
         no_hap_inds = []
-        #ref_haps = ref_info['Haplogroup'].tolist()
+
         # get freq of all total matched haplogroups
         ref_haps = sorted(Counter(ref_info['Haplogroup']).items())
 
-        base_cols = list(range(0, 9)) + list(range(-10, 0))
+        base_cols = list(range(0, 9)) + list(range(-11, 0))
         base_info = matched_info[:, base_cols]
-        key_matched_info = matched_info[matched_info[:,-3]=='Key']
+        key_matched_info = matched_info[matched_info[:,-4]=='Key']
 
         # traverse all indivisual to classify haplogroups
         final_list = []
@@ -352,6 +364,7 @@ class TrackLineage(object):
             global all_matched_haps
             # [1:] means remove match marker
             all_matched_haps = sorted(list(map(lambda x: x[1:], list(filter(lambda x: x[0]==self.matched_marker, col_info)))))
+
             # whether there are individual who don't have matched haplogroups
             if len(all_matched_haps) == 0:
                 no_hap_inds.append(ind)
@@ -359,7 +372,7 @@ class TrackLineage(object):
                 key_list.append('.')
                 mutation_list.append('.')
                 track_list.append('.')
-                self.logger.info('[Y-LineageTracker] [Warning] There are no mutation matched for sample %s' % (ind))
+                self.logger.warning('[Y-LineageTracker] [Warning] There are no mutation matched for sample %s' % (ind))
                 continue
 
             # extract matched haplogroups and their frequence
@@ -375,7 +388,8 @@ class TrackLineage(object):
                 else: # there are key haplogroups but not classified
                     key_hap, final_hap = self._detect_low_resolution_haplogroup(zipped_haps, matched_haps, key_haps, final_hap, key_hap)
             else: # no key haplogroup matched
-                self.logger.warning('[Y-LineageTracker] [Warning] There are no key haplogroups matched for sample %s' % (ind))
+                if self.build != 'Ref':
+                    self.logger.warning('[Y-LineageTracker] [Warning] There are no key haplogroups matched for sample %s' % (ind))
                 final_hap = ''
                 key_hap, final_hap = self._detect_low_resolution_haplogroup(zipped_haps, matched_haps, key_haps, final_hap)
 
@@ -387,12 +401,12 @@ class TrackLineage(object):
 
             # match all matched haplogroups to their mutations
             matched_haps_array = base_info[list(map(lambda x: x[0] == self.matched_marker, col_info))]
-            mutations = ', '.join(matched_haps_array[:, -10])
+            mutations = ', '.join(matched_haps_array[:, -11])
 
             # match final haplogroup to its mutations
-            final_hap_array = matched_haps_array[matched_haps_array[:, -9] == final_hap]
-            final_mutations = ', '.join(final_hap_array[:, -10])
-            if 'Key' not in final_hap_array[:, -3]:
+            final_hap_array = matched_haps_array[matched_haps_array[:, -10] == final_hap]
+            final_mutations = ', '.join(final_hap_array[:, -11])
+            if 'Key' not in final_hap_array[:, -4]:
                 final_mutations = '*' + final_mutations
 
             # get lineage track of haplogroup
@@ -407,6 +421,8 @@ class TrackLineage(object):
             track_list.append(track)
             if mutation:
                 all_mutation_list.append(mutations)
+            if final_hap in self.approximate_haps:
+                final_hap = final_hap + '~'
 
             final_list.append(final_hap)
             key_list.append(key_hap)
