@@ -1,3 +1,5 @@
+import os
+os.environ['QT_QPA_PLATFORM']='offscreen'
 import gc
 import sys
 import time
@@ -23,7 +25,6 @@ For polytomies, if sequence alignment is provided, the program will apply alignm
 
 
 config = getConfig()
-import_flag = True
 
 
 def phylo_parser():
@@ -49,7 +50,7 @@ def phylo_parser():
                         required=False,
                         type=str,
                         dest='format',
-                        choices=['vcf', 'fasta', 'phylip', 'nexus'],
+                        choices=['fasta', 'phylip', 'nexus', 'meg', 'vcf'],
                         help='seq-format: The file format of sequence alignment file. The default is fasta. ')
     # optional, population file
     parser.add_argument('-p', '--population',
@@ -63,7 +64,7 @@ def phylo_parser():
                         type=str,
                         dest='align',
                         action='store',
-                        default='ibs',
+                        default='mp',
                         choices=['ibs', 'upgma', 'mp'],
                         help='align-method: The method of constructing the bifurcating tree from multiple sequences. The default is mp.')
     # optional, the output format in text of tree
@@ -132,7 +133,7 @@ def check_phylo_output(tree_format, output):
 
     from FilesIO import get_out_path
     path = get_out_path(output, 'Phylo')
-    fig_file = path + '.phylo.fig,pdf'
+    fig_file = path + '.phylo'
     if 'xml' in tree_format:
         tree_file = path + '.tree.xml'
     else:
@@ -468,7 +469,6 @@ class PhyloHaplogroup(object):
     # get sub-tree from multiple sequence alignment
     def _get_sequence_distance_tree(self, seqs, ids, align):
 
-        global import_flag
         new_seqs = []
         # prune sequences
         for i in range(len(seqs[0])):
@@ -489,9 +489,8 @@ class PhyloHaplogroup(object):
 
         # IBS method
         if align == 'ibs':
-            if import_flag:
-                from scipy.cluster import hierarchy
-                from scipy.spatial.distance import squareform
+            from scipy.cluster import hierarchy
+            from scipy.spatial.distance import squareform
 
             pairwise_df = pd.DataFrame(index=ids, columns=ids)
 
@@ -513,13 +512,12 @@ class PhyloHaplogroup(object):
             dm_tree = hierarchy.to_tree(dm, False)
             children_tree = self._hier_to_newick(dm_tree, '', pairwise_df.index.tolist())
         else:
-            if import_flag:
-                import tempfile
-                from Bio import Phylo
-                from Bio.Seq import Seq
-                from Bio.SeqRecord import SeqRecord
-                from Bio.Align import MultipleSeqAlignment
-                from Bio.Phylo.TreeConstruction import DistanceCalculator, DistanceTreeConstructor
+            import tempfile
+            from Bio import Phylo
+            from Bio.Seq import Seq
+            from Bio.SeqRecord import SeqRecord
+            from Bio.Align import MultipleSeqAlignment
+            from Bio.Phylo.TreeConstruction import DistanceCalculator, DistanceTreeConstructor
 
             new_seqs = [''.join(i) for i in new_seqs]
             seq_list = [SeqRecord(Seq(j), id=i, name=i) for i, j in zip(ids, new_seqs)]
@@ -540,8 +538,7 @@ class PhyloHaplogroup(object):
             # MP (maximum parsimony) method
             elif align == 'mp':
 
-                if import_flag:
-                    from Bio.Phylo.TreeConstruction import ParsimonyScorer, NNITreeSearcher, ParsimonyTreeConstructor
+                from Bio.Phylo.TreeConstruction import ParsimonyScorer, NNITreeSearcher, ParsimonyTreeConstructor
 
                 dm = calculator.get_distance(aln)
                 constructor = DistanceTreeConstructor(calculator, 'nj')
@@ -556,8 +553,6 @@ class PhyloHaplogroup(object):
                 ete_tree = Tree(open(tree_tmp_file.name).read(), format=1)
                 children_tree = ete_tree.write(format=9)
 
-        import_flag = False
-
         return children_tree
 
     # IBS calculation
