@@ -91,18 +91,82 @@ class ReadFilterData(object):
 
         return sample_motif_count
 
+    def _get_specific_sites(self, build, sub_ref_info, id_info_dict):
+
+
+        data_info_array = []
+        for n, pos_num in enumerate(sub_ref_info['Build'+str(build)]):
+            line = []
+            int_pos = int(pos_num)
+            for bam, contig in id_info_dict.values():
+                pos_info = bam.fetch(contig, int_pos-1, int_pos)
+                pos_list = [i.seq[int_pos-i.pos-1].upper() for i in pos_info if int_pos-i.pos < i.qlen]
+                if len(set(pos_list)) == 1:
+                    pos = pos_list[0]
+                elif len(set(pos_list)) > 1:
+                    pos = Counter(pos_list).most_common()[0][0]
+                else:
+                    pos = '.'
+                line.append(pos)
+            data_info_array.append([pos_num] + line)
+
+        data_info = pd.DataFrame(columns=['POS']+list(id_info_dict.keys()), data=np.array(data_info_array))
+
+        return data_info
+
+    def _match_sites(self, data_info, ref_info, build, matched_marker, header_cut, header_end):
+
+        matched_info = pd.merge(data_info, ref_info,
+                                left_on='POS',
+                                right_on='Build'+str(build),
+                                how='inner',
+                                sort=False)
+        matched_info_array = np.array(matched_info)
+        hap_function = (lambda x: matched_marker+matched_info_array[i, -10] if x == match_marker else x)
+        for i in range(len(matched_info)):
+            match_marker = matched_info_array[i, -6]
+            matched_info_array[i, header_cut: header_end] = list(map(hap_function, matched_info_array[i, header_cut: header_end]))
+
+        final_matched_info = pd.DataFrame(columns=matched_info.columns.tolist(), data=matched_info_array)
+
+        return final_matched_info
+
+    def _get_female_by_KMeans(self, missing_rate_data, samples):
+
+        from sklearn.cluster import KMeans
+
+        missing_rate_data.extend([0, 1])
+        missing_rate_data = np.array(missing_rate_data).reshape(-1, 1)
+        estimator = KMeans(n_clusters=2)
+        res = estimator.fit_predict(missing_rate_data)[: -2]
+        centroids = estimator.cluster_centers_
+        if centroids[0][0] > centroids[1][0]:
+            del_list = [samples[j] for i, j in zip(res, range(len(res))) if i == 0]
+            female_num = len(list(filter(lambda x: x == 0, res)))
+        else:
+            del_list = [samples[j] for i, j in zip(res, range(len(res))) if i == 1]
+            female_num = len(list(filter(lambda x: x == 1, res)))
+
+        return del_list, female_num
+
     # read BAM and do matching analysis
-    def read_bam(self, ref_info, build, extract_type):
+    def read_bam(self, ref_info, build, extract_type, format):
 
         import os
         import pysam
         from collections import Counter
+
+        if format == 'bam':
+            open_method = 'rb'
+        else:
+            open_method = 'rc'
+
         # get bam list
         input_file_list = self.input_file.split(', ')
         if len(input_file_list) == 1:
             file = input_file_list[0]
             try:
-                pysam.AlignmentFile(file)
+                pysam.AlignmentFile(file, open_method)
                 bam_list = input_file_list
             except ValueError:
                 bam_list = open(self.input_file).read().splitlines()
@@ -113,18 +177,20 @@ class ReadFilterData(object):
                 if not file_exists:
                     raise FileNotFoundError('No such file or directory: %s' % i)
 
-        inds = []
-        contigs = []
-        bams = []
+        id_info_dict = {}
         format_contigs = config.get('DataInfo', 'Contigs').split(',')
         for f in bam_list:
-            bam = pysam.AlignmentFile(f)
+            file_name = os.path.basename(f)
+            bam = pysam.AlignmentFile(f, open_method)
             bam_with_index = bam.has_index()
             if not bam_with_index:
-                self.logger.error('[Y-LineageTracker] bam file %s has no index' % f)
+                self.logger.error('[Y-LineageTracker] bam/cram file %s has no index' % f)
                 sys.exit()
+            ind = bam.header.get('RG')[0]['ID']
+            if ind in self.samples:
+                continue
             sample_contigs = [i.contig for i in bam.get_index_statistics()]
-            overlaped_contigs = list(set(set(sample_contigs) & set(format_contigs)))
+            overlaped_contigs = list(set(sample_contigs) & set(format_contigs))
             if len(overlaped_contigs) == 1:
                 contig = overlaped_contigs[0]
             else:
@@ -134,46 +200,91 @@ class ReadFilterData(object):
                 if len(overlaped_contigs) == 0:
                     self.logger.error('[Y-LineageTracker] No contig for Y chromosome (Y, chrY, 24, chr24), please check input data')
                     sys.exit()
-            inds.append(bam.header.get('RG')[0]['ID'])
-            contigs.append(contig)
-            bams.append(bam)
+            id = ind+'('+file_name+')'
+            id_info_dict[id] = [bam, contig]
 
         # for NRY haplogroup classification
         # match ISOGG sites to BAM files
         if extract_type == 'snp':
-            data_info_array = []
-            for n, pos_num in enumerate(ref_info['Build'+str(build)]):
-                line = []
-                int_pos = int(pos_num)
-                for bam, contig in zip(bams, contigs):
-                    pos_info = bam.fetch(contig, int_pos-1, int_pos)
-                    pos_list = [i.seq[int_pos-i.pos-1].upper() for i in pos_info if int_pos-i.pos < i.qlen]
-                    if len(set(pos_list)) == 1:
-                        pos = pos_list[0]
-                    elif len(set(pos_list)) > 1:
-                        pos = Counter(pos_list).most_common()[0][0]
-                    else:
-                        pos = '.'
-                    line.append(pos)
+            # match sites of main trunk
+            matched_marker = config.get('HaplogroupMarker', 'MatchMarker') # used to mark matched haplogroups
+            header_cut = config.getint('DataInfo', 'BamHeaderCut')
+            header_end = -config.getint('DataInfo', 'ISOGGInfoCut')
+            common_trunk = config.get('HaplogroupTree', 'CommonTrunk').split(',')
+            special_haps = config.get('HaplogroupTree', 'SpecialTrunk').split(',')
 
-                data_info_array.append([pos_num] + line)
-                percent = float(n + 1) * 100 / float(ref_info.index.size)
-                sys.stdout.write('[Y-LineageTracker] Extrating positions and variants from BAMs... %.2f' % percent)
-                sys.stdout.write('%\r')
-                sys.stdout.flush()
-            data_info = pd.DataFrame(columns=['POS']+inds, data=np.array(data_info_array))
-            if self.samples:
-                used_males = self._get_male_inds(inds)
-                data_info = data_info[['POS']+used_males]
-            print()
-            self.logger.info('[Y-LineageTracker] Extrating information from BAMs finifhsed')
+            # get allele of haplogroup on main trunks
+            main_trunk_ref_info = ref_info[ref_info['MainInfo']=='Main']
+            self.logger.info('[Y-LineageTracker] Extrating mian trunks from BAMs/CRAMs...')
+            main_trunk_info = self._get_specific_sites(build, main_trunk_ref_info, id_info_dict)
+            # filter females
+            main_marker_num = main_trunk_info.index.size
+            f_samples = main_trunk_info.columns.tolist()[1:]
+            missing_rate_data = [main_trunk_info[col].tolist().count('.')/main_marker_num for col in f_samples]
+            del_list, female_num = self._get_female_by_KMeans(missing_rate_data, f_samples)
+            if del_list:
+                if female_num == len(f_samples):
+                    self.logger.error('[Y-LineageTracker] Program stopped since no male sample left for subsequent analysis')
+                    sys.exit()
+                else:
+                    main_trunk_info.drop(del_list, inplace=True, axis=1)
+                    for female in del_list:
+                        del id_info_dict[female]
+
+            # match main haplogroups
+            main_matched_info = self._match_sites(main_trunk_info, main_trunk_ref_info, build, matched_marker, header_cut, header_end)
+            self.logger.info('[Y-LineageTracker] Extrating mian trunks from BAMs/CRAMs finifhsed')
+
+            # find which main haplogroup with higher matching rate
+            main_sub_info = main_matched_info[main_matched_info['Haplogroup'].map(lambda x: x not in common_trunk)]
+            all_haps = main_sub_info['Haplogroup'].tolist()
+            main_sub_info = np.array(main_sub_info)
+            id_hap_dict = {}
+            for id_num, id in enumerate(id_info_dict.keys()):
+                col_info = main_sub_info[:, id_num+header_cut]
+                id_haps = sorted(list(map(lambda x: x[1:], list(filter(lambda x: x[0]==matched_marker, col_info)))))
+                ratio = 0
+                for hap in set(id_haps):
+                    ratio2 = id_haps.count(hap) / all_haps.count(hap)
+                    if ratio2 > ratio:
+                        ratio = ratio2
+                        main_hap = hap
+                if ratio < 0.25:
+                    main_hap = None
+                id_hap_dict[id] = main_hap
+
+            # extract sub-haplogroups of each main trunk
+            self.logger.info('[Y-LineageTracker] Extrating sub-trunks from BAMs/CRAMs...')
+            sub_trunk_ref_info = ref_info[ref_info['MainInfo']!='Main']
+            info_list = []
+            NA_ids = []
+            for hap in sorted(set(id_hap_dict.values())):
+                hap_ids = list({k: v for k, v in id_hap_dict.items() if v==hap}.keys())
+                sub_id_info_dict = {k: v for k, v in id_info_dict.items() if k in hap_ids}
+                if main_hap:
+                    hap_ref_info = sub_trunk_ref_info[sub_trunk_ref_info['Haplogroup'].map(lambda x: x.startswith(hap))]
+                    hap_info = self._get_specific_sites(build, hap_ref_info, sub_id_info_dict)
+                    info_list.append(hap_info)
+                else:
+                    hap_ref_info = sub_trunk_ref_info[sub_trunk_ref_info['Haplogroup'].map(lambda x: x.startswith('A') and x in special_haps)]
+                    hap_info = self._get_specific_sites(build, hap_ref_info, sub_id_info_dict)
+            sub_hap_info = pd.concat(info_list, axis=0, sort=False).fillna('.')
+            self.logger.info('[Y-LineageTracker] Extrating sub-trunks from BAMs/CRAMs finifhsed')
+
+            sub_trunk_info = sub_hap_info[['POS']+list(id_info_dict.keys())]
+            sub_matched_info = self._match_sites(sub_trunk_info, sub_trunk_ref_info, build, matched_marker, header_cut, header_end)
+
+            data_info = np.array(pd.concat([main_matched_info, sub_matched_info]).drop_duplicates())
+            inds = list(id_info_dict.keys())
+
+            self.logger.info('[Y-LineageTracker] Extrating information from BAMs/CRAMs finifhsed')
 
         # for Y-STR genotyping
         # genotype Y-STRs from BAMs
         else:
             from FilesIO import get_str_from_panel
             str_info = get_str_from_panel(ref_info)
-            data_info = pd.DataFrame(columns=inds)
+            data_info = pd.DataFrame(columns=list(id_info_dict.keys()))
             for num, STR_idx in enumerate(str_info.index):
                 # read basic information
                 start = str_info.at[STR_idx, 'Start'+str(build)]
@@ -184,7 +295,7 @@ class ReadFilterData(object):
                 end_motif = str_info.at[STR_idx, 'End']
                 motif_count = []
                 # matching
-                for bam, contig in zip(bams, contigs):
+                for bam, contig in id_info_dict.values():
                     base_motif_num = str_info.at[STR_idx, 'Ref'+str(build)]
                     sample_motif_count = self._count_motif_num(bam, contig, start, end, base_motif_num, motif, motif_length, end_motif)
                     motif_count.append(sample_motif_count)
@@ -194,9 +305,7 @@ class ReadFilterData(object):
                 sys.stdout.write('%\r')
                 sys.stdout.flush()
             data_info = data_info.T
-            if self.samples:
-                used_males = self._get_male_inds(inds)
-                data_info = data_info.loc[used_males]
+            inds = list(id_info_dict.keys())
 
             print()
             self.logger.info('[Y-LineageTracker] Genotyping Y-STR finished')
@@ -262,7 +371,6 @@ class ReadFilterData(object):
     # filter individuals
     def _filter_variant(self, data, cutoff, file_format, header_cut):
 
-        from sklearn.cluster import KMeans
         # list of indivisuals and number
         inds = data.columns.tolist()[header_cut:]
         ind_num_before = len(inds)
@@ -294,20 +402,10 @@ class ReadFilterData(object):
                         female_num = len([i for i in inds if i not in del_list])
                     # if not set cutoff, will use K-means to cluster missing rate to identify samples and females
                     else:
-                        missing_rate_data.extend([0, 1])
-                        missing_rate_data = np.array(missing_rate_data).reshape(-1, 1)
-                        estimator = KMeans(n_clusters=2)
-                        res = estimator.fit_predict(missing_rate_data)[: -2]
-                        centroids = estimator.cluster_centers_
-                        if centroids[0][0] > centroids[1][0]:
-                            del_list = [inds[j] for i, j in zip(res, range(len(res))) if i == 0]
-                            female_num = len(list(filter(lambda x: x == 0, res)))
-                        else:
-                            del_list = [inds[j] for i, j in zip(res, range(len(res))) if i == 1]
-                            female_num = len(list(filter(lambda x: x == 1, res)))
+                        del_list, female_num = self._get_female_by_KMeans(missing_rate_data, inds)
                     # if all individual filtered
                     if len(del_list) == ind_num_before:
-                        self.logger.error('[Y-LineageTracker] Program stopped since no sample left for subsequent analysis')
+                        self.logger.error('[Y-LineageTracker] Program stopped since no male sample left for subsequent analysis')
                         sys.exit()
                     # remove filtered indivisuals from data
                     data.drop(del_list, inplace=True, axis=1)
