@@ -403,7 +403,8 @@ class StatisticsAnalysis(object):
 
         # calculate MSP and MSG to get theta statistic
         pops_freq = [np.array(i) / j for i, j in zip(pops_count, sample_sizes)]
-        mean_freq = np.sum([np.array(i) for i in pops_count], axis=0) / np.sum(sample_sizes)
+        mean_freq = np.sum([np.array(i) for i in pops_count]) / np.sum(sample_sizes)
+
 
         MSP = np.sum([i*((j-mean_freq)**2) for i, j in zip(sample_sizes, pops_freq)], axis=0)
 
@@ -413,12 +414,12 @@ class StatisticsAnalysis(object):
 
         nc = np.sum(sample_sizes) - np.sum(sample_sizes**2)/np.sum(sample_sizes)
 
-        theta1 = np.nansum(MSP-MSG)
-        theta2 = np.nansum(MSP+(nc-1)*MSG)
+        theta1 = np.nansum(MSP)-np.nansum(MSG)
+        theta2 = np.nansum(MSP)+(nc-1)*np.nansum(MSG)
         if theta2 == 0:
             theta = 0
         else:
-            theta = theta1 / theta2
+            theta = (theta1 / theta2)
 
         return theta
 
